@@ -8,13 +8,12 @@
 #   the GNU Lesser General Public License version 2 or later.
 #
 
-require 'amstd/fileutils'
+require 'fileutils'
 
 
 module TMail
 
   class Port
-    # initialize() may defined by user
   end
 
 
@@ -25,14 +24,16 @@ module TMail
   class FilePort < Port
 
     def initialize( fname )
-      @filename = File.expand_path( fname )
+      @filename = File.expand_path(fname)
       super()
     end
 
-    attr :filename
+    attr_reader :filename
+
+    alias ident filename
 
     def eql?( other )
-      FilePort === other and @filename == other.filename
+      other.respond_to? :filename and @filename == other.filename
     end
 
     alias == eql?
@@ -105,13 +106,12 @@ module TMail
     end
 
     def mv( port )
-      FILEUTILS.mv @filename, port.filename
-      rm
+      FileUtils.mv @filename, port.filename
     end
 
     def cp( port )
       if Port === port then
-        FILEUTILS.cp @filename, port.filename
+        FileUtils.cp @filename, port.filename
       else
         ropen {|f|
           while tmp = f.read(4096) do
@@ -124,6 +124,125 @@ module TMail
   end
 
 
+  module MailFlags
+
+    def seen=( b )
+      set_status 'S', b
+    end
+
+    def seen?
+      get_status 'S'
+    end
+
+    def replied=( b )
+      set_status 'R', b
+    end
+
+    def replied?
+      get_status 'R'
+    end
+
+    def flagged=( b )
+      set_status 'F', b
+    end
+
+    def flagged?
+      get_status 'F'
+    end
+
+    def procinfostr( str, tag, true_p )
+      a = str.upcase.split(//)
+      a.push true_p ? tag : nil
+      a.delete tag unless true_p
+      a.compact.sort.join('').squeeze
+    end
+  
+  end
+
+
+  class MhPort < FilePort
+
+    include MailFlags
+
+    private
+    
+    def set_status( tag, flag )
+      begin
+        tmpfile = @filename + '.tmailtmp.' + $$.to_s
+        File.open( tmpfile, 'w' ) {|f|
+          write_status f, tag, flag
+        }
+        File.unlink @filename
+        File.link tmpfile, @filename
+      ensure
+        File.unlink tmpfile
+      end
+    end
+
+    def write_status( f, tag, flag )
+      stat = ''
+      File.open( @filename ) {|r|
+        while line = r.gets do
+          if line.strip.empty? then
+            break
+          elsif m = /\AX-Delta-Status:/i.match(line) then
+            stat = m.post_match.strip
+          else
+            f.print line
+          end
+        end
+
+        s = procinfostr(stat, tag, flag)
+        f.puts 'X-Delta-Status: ' + s unless s.empty?
+        f.puts
+
+        while s = r.read(2048) do
+          f.write s
+        end
+      }
+    end
+
+    def get_status( tag )
+      File.foreach( @filename ) {|line|
+        return false if line.strip.empty?
+        if m = /\AX-Delta-Status:/i.match(line) then
+          return m.post_match.strip.include? tag[0]
+        end
+      }
+      false
+    end
+  
+  end
+
+
+  class MaildirPort < FilePort
+
+    include MailFlags
+
+    RE = /\A(\d+\.\d+.[^:]+)(?:\:(\d),(\w+)?)?\z/
+
+    def set_status( tag, flag )
+      unless m = RE.match(File.basename @filename) then
+        newname = @filename + ':2,' + tag
+      else
+        s, uniq, type, info, = m.to_a
+        return if type and type != '2'  # do not change anything
+        newname = File.dirname(@filename) + '/' +
+                  uniq + ':2,' + procinfostr(info.to_s, tag, flag)
+      end
+
+      File.link @filename, newname
+      File.unlink @filename
+      @filename = newname
+    end
+
+    def get_status( tag )
+      m = RE.match(File.basename @filename) or return false
+      m[2] == '2' and m[3].to_s.include? tag[0]
+    end
+  
+  end
+
 
   class Stream
 
@@ -180,9 +299,9 @@ module TMail
       @filename = nil
     end
 
-    def each
+    def each( &block )
       setterm unless @term
-      @f.each( @term ){|l| yield l }
+      @f.each( @term, &block )
     end
 
     def gets
@@ -311,7 +430,7 @@ module TMail
       super()
     end
 
-    attr :str; protected :str
+    attr :str
 
     def size
       @str.size
@@ -436,7 +555,7 @@ module TMail
 
     def each( &block )
       setterm unless @term
-      @str.each( @term ) {|s| yield s }
+      @str.each( @term, &block )
     end
 
     def gets
